<?php

/**
 * @file
 * Test case for Juicebox file handling.
 */

namespace Drupal\juicebox\Tests;

use Drupal\Component\Utility\String;
use Drupal\views\Tests\ViewTestData;


/**
 * Tests the Juicebox XML generation via a sub-request.
 *
 * @group Juicebox
 */
class JuiceboxSubRequestCase extends JuiceboxBaseCase {

  // For some totally vexing (and downright damming) reason, tests using views
  // that contain file/image fields won't work if the field configuration is
  // created progrmatically as part of the test itself. In other words if we
  // use JuiceboxBaseCase::initNode() to add fields and attach them to a node,
  // and then load a view conf that uses these fields, things won't work
  // (the view will have broken handlers). Surely there is some cache clearing
  // technique that will get around this? Anyway, after hours of bloody test
  // debugging I have resourted to loading the field conf within a helper module
  // (juicebox_mimic_article) as it seems to get around this. That module just
  // mimics an "article" type so another option is to use the "standard" profile
  // and remove juicebox_mimic_article below (but that's slow).
  public static $modules = array('node', 'field', 'image', 'juicebox', 'views', 'juicebox_mimic_article', 'juicebox_test_views');
  protected $instBundle = 'article';
  protected $instFieldName = 'field_image';
  // Uncomment the line below, and remove juicebox_mimic_article from the module
  // list above, to use the "standard" profile's article type for this test
  // instead of the one we create manually (should also work, but will be slow).
  // protected $profile = 'standard';

  // Views used by this test.
  public static $testViews = array('juicebox_test_row_formatter');


  /**
   * Define setup tasks.
   */
  public function setUp($import_test_views = TRUE) {
    parent::setUp($import_test_views);
    // Create and login user.
    $this->webUser = $this->drupalCreateUser(array('access content', 'access administration pages', 'administer site configuration', 'administer content types', 'administer nodes', 'bypass node access'));
    $this->drupalLogin($this->webUser);
    // Create a test node. Note that we don't need to initiate a node and field
    // structure before this because that's been handled for us by
    // juicebox_mimic_article.
    $this->createNodeWithFile('image', FALSE, FALSE);
    // Enable our test views.
    if ($import_test_views) {
      ViewTestData::createTestViews(get_class($this), array('juicebox_test_views'));
    }
  }

  /**
   * Test a gallery embedded in a view row that is dependent on the Juicebox
   * cache.
   */
  public function testSubRequestDependent() {
    $node = $this->node;
    $xml_path = 'juicebox/xml/field/node/' . $node->id() . '/' . $this->instFieldName . '/_custom';
    // Get the urls to the test image and thumb derivative used by default.
    $uri = \Drupal\file\Entity\File::load($node->{$this->instFieldName}[0]->target_id)->getFileUri();
    $test_image_url = entity_load('image_style', 'juicebox_medium')->buildUrl($uri);
    $test_thumb_url = entity_load('image_style', 'juicebox_square_thumb')->buildUrl($uri);
    // Check for correct embed markup. This will also prime the cache.
    $content = $this->drupalGet('juicebox_test_row_formatter');
    $this->assertRaw("new juicebox({configUrl:'" . \Drupal::url('juicebox.xml_field', array('args' => str_replace('juicebox/xml/', '', $xml_path))), 'Embed javascript found.');
    $this->assertRaw('id="field--node--' . $node->id() . '--' . str_replace('_', '-', $this->instFieldName) . '---custom"', 'Embed code wrapper found.');
    $this->assertRaw(String::checkPlain($test_image_url), 'Test image found in embed code');
    // Extract the xml-source values from the XML.
    $matches = array();
    preg_match("/xml-source-path=([^?&']+).*xml-source-id=([^?&']+)/", $content, $matches);
    $this->assertNotNull($matches[1], 'xml-source-path value found in embed javascript.');  
    $this->assertNotNull($matches[2], 'xml-source-id value found in embed javascript.');
    // Check for correct XML. This example is dependent on a sub-request XML
    // lookup, so everything below would fail without that feature.
    $this->drupalGet($xml_path, array('query' => array('xml-source-path' => $matches[1], 'xml-source-id' => $matches[2])));
    $this->assertRaw('<?xml version="1.0" encoding="UTF-8"?>', 'Valid XML detected.');
    $this->assertRaw('imageURL="' . String::checkPlain($test_image_url), 'Test image found in XML.');
    $this->assertRaw('thumbURL="' . String::checkPlain($test_thumb_url), 'Test thumbnail found in XML.');
    $this->assertRaw('backgroundcolor="green"', 'Custom background setting from pseudo field instance config found in XML.');
  }

}
